//==============================================================================
// WPInfo WinPoint 95 sample Plugin          Copyright (C) 1998 Tim Schattkowsky
//==============================================================================
// LastRevision:
//   11.6.1998
//==============================================================================
// Description:
//   A simple Plugin that displays version informatin about the WinPoint host.
//   This plugin also demonstrates a way to hide the Plugins main window from
//   the Taskbar and screen.
//==============================================================================
unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WinpointAPI, WPAPICommon, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    WinPointAPI: TWinPointAPI;
    WinPointInfo: TWinPointService;
    Bevel1: TBevel;
    WPName: TLabel;
    WPCopyright: TLabel;
    Button1: TButton;
    Label1: TLabel;
    WPVersion: TLabel;
    Label2: TLabel;
    WPDesc: TLabel;
    procedure WinPointAPIServerDisconnected(Sender: TObject);
    procedure WinPointInfoCommand(Msg: TMsg);
    procedure Button1Click(Sender: TObject);
    procedure WinPointAPIConnectFailed(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure WinPointAPIWrongVersion(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    FirstShow :Boolean;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.WinPointAPIServerDisconnected(Sender: TObject);
begin
     // Quit because WinPoint did it
     Application.Terminate;
end;

procedure TForm1.WinPointAPIConnectFailed(Sender: TObject);
begin
     // Connect Failed
     ShowMessage('Could not connect to WinPoint 95!');
     Application.Terminate;
end;

procedure TForm1.WinPointAPIWrongVersion(Sender: TObject);
begin
     // Version negociation failed
     ShowMessage('This Plugin works only with a legal copy of WinPoint 95 '+
                 'Release 1.0 or above!');
end;

procedure TForm1.WinPointInfoCommand(Msg: TMsg);
var
  AppInfo:TAppInfo;
begin
     // User selected the Service !
     WinPointAPI.GetInfo(AppInfo);
     with APPInfo do
     begin
          WPName.Caption      :=Name;
          WPCopyright.Caption :=Copyright;
          WPVersion.Caption   :=IntToStr(Version)+'.'+IntToStr(Revision);
          WPDesc.Caption      :=Description;
          WindowState         :=wsNormal;
          Show;
          Application.BringToFront;
     end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     // Hide Window from Taskbar
     ShowWindow(Application.Handle,SW_HIDE);
     // Indicate that the next Paint will be the first ever ;)
     FirstShow :=True;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
     // Hide the Window an Startup
     if FirstShow then
     begin
          FirstShow :=False;
          Hide;
     end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
     // Hide Info Window
     Hide;
end;

end.
