//==============================================================================
// A Delphi Class to handle Fido-Style addresses
//==============================================================================
//
// (C) 1998-1999 by Tim Schattkowsky
//
// Last Revision: 03.04.1999  
//
//==============================================================================
// Changes
//==============================================================================
// 02.07.1998  Those ugly class functions are now normal functions.
//             New method Assign
//             New event OnChange
// 03.04.1999  New Emtpy and EmptyText properties for TFidonetAddress
//             New method Clear
//==============================================================================
unit Fidoaddr;

interface

uses
  Classes,SysUtils;

type
  TPackedAddress = packed record
    Zone :Word;
    Net  :Word;
    Node :Word;
    Point:Word;
  end;
  {============================================================================}
  {=== TFidonetAddress ========================================================}
  {============================================================================}
  TFidonetAddress = class
  private
    // Property variables
    FNText      :String;
    FOnChange   :TNotifyEvent;
    FPacked     :TPackedAddress;
    FText       :String;
    // Property access
    function    REmpty:Boolean;
    function    REmptyText:String;
    procedure   WAddress(AAddr:TPackedAddress);
    procedure   WNet(ANet:Word);
    procedure   WNode(ANode:Word);
    procedure   WPoint(APoint:Word);
    procedure   WText(AText:String);
    procedure   WZone(AZone:Word);
    // Private methods
    procedure   MakeText;
  protected
    procedure   Change; virtual;
  public
    constructor Create;
    // Methods
    procedure   Assign(AAddr:TFidonetAddress);
    procedure   Clear;
    function    Equals(AAddr:TPackedAddress):Boolean;
    // Properties
    property    Address:TPackedAddress read FPacked       write WAddress;
    property    Empty:Boolean          read REmpty;
    property    EmptyText:String       read REmptyText    write WText;
    property    Net:Word               read FPacked.Net   write WNet;
    property    Node:Word              read FPacked.Node  write WNode;
    property    NodeText:String        read FNText;
    property    Point:Word             read FPacked.Point write WPoint;
    property    Text:String            read FText         write WText;
    property    Zone:Word              read FPacked.Zone  write WZone;
    // Events
    property    OnChange:TNotifyEvent  read FOnChange     write FOnChange;
  end;

function  AddrEq(A,B:TPackedAddress):Boolean;
function  AddrGa(A,B:TPackedAddress):Boolean;
function  AddrToStr(AAddr:TPackedAddress):String;
procedure StrToAddr(AString:String; var A:TPackedAddress);

implementation

function AddrToStr(AAddr:TPackedAddress):String;
begin
     with AAddr do
     begin
          AddrToStr:=IntToStr(Zone)+':'+IntToStr(Net)+'/'+IntToStr(Node);
          if Point>0 then Result:=Result+'.'+IntToStr(Point);
     end;
end;

procedure StrToAddr(AString:String; var A:TPackedAddress);
var NewZone  :String;
    NewNet   :String;
    NewNode  :String;
    NewPoint :String;
    I        :Integer;
begin
     NewZone  :='';
     I        :=1;
     AString  :=AString+#0;                     {Terminate it!}
     if AString[1] in ['0'..'9'] then
     begin
          while AString[I] in ['0'..'9'] do
          begin
               NewZone:=NewZone+AString[I];
               Inc (I);
          end;
          if AString[I]=':' then
          if AString[I+1] in ['0'..'9'] then
          begin
               Inc (I);
               NewNet:='';
               while AString[I] in ['0'..'9'] do
               begin
                    NewNet:=NewNet+AString[I];
                    Inc (I);
               end;
               if AString[I]='/' then
               if AString[I+1] in ['0'..'9'] then
               begin
                    Inc (I);
                    NewNode:='';
                    while AString[I] in ['0'..'9'] do
                    begin
                         NewNode:=NewNode+AString[I];
                         Inc (I);
                    end;
                    with A do
                    if AString[I]='.' then
                    begin
                         if AString[I+1] in ['0'..'9'] then
                         begin
                              Inc (I);
                              NewPoint:='';
                              while AString[I] in ['0'..'9'] do
                              begin
                                   NewPoint:=NewPoint+AString[I];
                                   Inc (I);
                              end;
                              Zone  :=StrToInt(NewZone);
                              Net   :=StrToInt(NewNet);
                              Node  :=StrToInt(NewNode);
                              Point :=StrToInt(NewPoint);
                         end;
                    end else
                    begin
                         Zone  :=StrToInt(NewZone);
                         Net   :=StrToInt(NewNet);
                         Node  :=StrToInt(NewNode);
                         Point :=0;
                    end;
               end;
          end;
     end;
end;

function AddrEq(A,B:TPackedAddress):Boolean;
begin
     with A do
     AddrEq:=((Zone=B.Zone)and(Net=B.Net)and(Node=B.Node)and(Point=B.Point));
end;

function AddrGA(A,B:TPackedAddress):Boolean;
begin
     with A do
     if Zone<B.Zone then Result:=False else
     if Zone>B.Zone then Result:=True  else
     if Net<B.Net   then Result:=False else
     if Net>B.Net   then Result:=True  else
     if Node<B.Node then Result:=False else
     if Node>B.Node then Result:=True  else Result:=(Point>B.Point);
end;

{==============================================================================}
{=== TFidonetAddress implementation ===========================================}
{==============================================================================}
constructor TFidonetAddress.Create;
begin
     inherited Create;
     with FPacked do
     begin
          Zone   :=0;
          Net    :=0;
          Node   :=0;
          Point  :=0;
     end;
     FNText :='0:0/0';
     FText  :=FNText;
end;

function TFidonetAddress.REmptyText:String;
begin
     if Empty then Result:='' else Result:=Text;
end;

function TFidonetAddress.REmpty:Boolean;
begin
     Result:=(Zone=0)and(Net=0)and(Node=0)and(Point=0);
end;

function TFidonetAddress.Equals(AAddr:TPackedAddress):Boolean;
begin
     Equals:=AddrEq(AAddr,FPacked);
end;

procedure TFidonetAddress.Assign(AAddr:TFidonetAddress);
begin
     Address:=AAddr.Address;
end;

procedure TFidonetAddress.Clear;
begin
     with FPacked do
     begin
          Zone   :=0;
          Net    :=0;
          Node   :=0;
          Point  :=0;
     end;
     Change;
end;

procedure TFidonetAddress.WAddress(AAddr:TPackedAddress);
begin
     if not AddrEq(FPacked,AAddr) then
     begin
          FPacked:=AAddr;
          Change;
     end;
end;

procedure TFidonetAddress.WZone(AZone:Word);
begin
     if FPacked.Zone<>AZone then
     begin
          FPacked.Zone:=AZone;
          Change;
     end;
end;

procedure TFidonetAddress.WNet(ANet:Word);
begin
     if FPacked.Net<>ANet then
     begin
          FPacked.Net:=ANet;
          Change;
     end;
end;

procedure TFidonetAddress.WNode(ANode:Word);
begin
     if FPacked.Node<>ANode then
     begin
          FPacked.Node:=ANode;
          Change;
     end;
end;

procedure TFidonetAddress.WPoint(APoint:Word);
begin
     if FPacked.Point<>APoint then
     begin
          FPacked.Point:=APoint;
          Change;
     end;
end;

procedure TFidonetAddress.WText(AText:String);
begin
     if AText<>FText then
     begin
          StrToAddr(AText,FPacked);
          Change;
     end;
end;

procedure TFidonetAddress.Change;
begin
     MakeText;
     if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TFidonetAddress.MakeText;
var
  Temp:Word;
begin
     with FPacked do
     begin
          Temp   :=Point;
          Point  :=0;
          FNText :=AddrToStr(FPacked);
          if Temp<>0 then
          begin
               Point :=Temp;
               FText :=AddrToStr(FPacked);
          end else FText:=FNText;
     end;
end;

end.
