//==============================================================================
// WinPoint 95 API common functions and declarations
//==============================================================================
//
// (C) 1998-1999 by Tim Schattkowsky
//
// API Version implemented: 9
// Last Revision: 28.04.1999
//
// NOTE: WinPoint 95 Release 1.6A or later is requiered !
//==============================================================================

unit WPAPICommon;

interface
uses
  Messages, Windows, SysUtils, FidoAddr;

const
  {============================================================================}
  {= Constants for API calls ==================================================}
  {============================================================================}
  WP_FIRST             = WM_USER+1;
  //----------------------------------------------------------------------------
  // Init Message - MUST be send to the WinPoint Main Window using SendMessage
  WP_QUERY_API         = WM_USER+1;
  // Handshaking messages
  WP_ACK               = WM_USER+2;
  WP_RESULT            = WM_USER+3;
  WP_UNSUPPORTED       = WM_USER+4;
  // WP API Messages - MUST be send to the APIs Thread using PostThreadMessage
  {--- API Version 1 ---}
  {These functions MUST be supported by both the Server and the Client! }
  WP_DISCONNECT        = WM_USER+5;   // Notify that the connection breaks
  WP_GETINFO           = WM_USER+6;   // Get information
  {These functions are supported by the Server}
  WP_GETAPIVERSION     = WM_USER+7;   // Get Version of WinPoint Plugin API
  WP_GETAREALIST       = WM_USER+8;   // Get a list of all available areas
  WP_OPENAREA          = WM_USER+9;   // Open an area
  WP_CLOSEAREA         = WM_USER+10;  // Close the currently opened area
  WP_GETMSGLIST        = WM_USER+11;  // Get Messagelist for currently open area
  WP_GETMSG            = WM_USER+12;  // Get Messagel from currently open area
  WP_REGISTERSERVICE   = WM_USER+13;  // Register a Plugin provided Service
  WP_DEREGISTERSERVICE = WM_USER+14;  // DeRegister a Plugin provided Service
  // Version 2
  WP_STOREMSG          = WM_USER+15;  // Store a msg in the current area
  // Version 3
  WP_QUIT              = WM_USER+16;  // Internal!!!!!!!!! - Dont use!
  // Version 4
  WP_GETMSGFLAGS       = WM_USER+17;  // Get Flags of Message
  WP_SETMSGFLAGS       = WM_USER+18;  // Set Flags of Message
  // Version 5
  WP_CREATEAREA        = WM_USER+19;  // Create a new Area
  // Version 6
  WP_GETCURRENTAREA    = WM_USER+20;  // Get currently active area
  // Version 7
  WP_GETAREAUPLINK     = WM_USER+21;  // Get Uplink for Area
  WP_GETUPLINKS        = WM_USER+22;  // Get all Uplinks
  WP_GETAREATYPE       = WM_USER+23;  // Get Type for Area
  // Version 8 (WinPoint Release 1.0)
  WP_XCMD              = WM_USER+24;  // Internal!!!!!!!!! - Dont use!
  // Version 9 (WinPoint Release 1.6A)
  WP_GETTRANSLATIONS   = WM_USER+25;
  WP_SETTRANSLATIONS   = WM_USER+26;
  //----------------------------------------------------------------------------
  WP_LAST              = WM_USER+26;

  WP_SERVICEBASE       = WM_USER+1024;
  WP_FIRSTSERVICE      = WM_USER+1024;
  WP_LASTSERVICE       = WM_USER+2048;

  // Generic return codes
  WP_SUCCESS             = $00000000;
  WP_FAILED              = $FFFFFFFF;
  // Common codes
  WPERR_COMMON           = $80001000;
  WPERR_NOPARAM          = WPERR_COMMON+1; // param passing for string cmd fail
  WPERR_NOAREAOPEN       = WPERR_COMMON+2; // no area open
  WPERR_WRONGFORMAT      = WPERR_COMMON+3; // obvious data format violation
  // Msg related Error Codes
  WPERR_MSG              = $80002000;
  WPERR_BADMSGINDEX      = WPERR_MSG+1;
  // Area related Error Codes
  WPERR_AREA             = $80003000;      // Area related error
  WPERR_AREAOPEN         = WPERR_AREA+1;   // the Area could not be opened
  WPERR_AREAUNKNOWN      = WPERR_AREA+2;   // the Areaname is invalid
  WPERR_AREAINUSE        = WPERR_AREA+3;   // the Area is in use
  WPERR_AREAEXISTS       = WPERR_AREA+4;   // the Area alreasy exists
  // Service related Error Codes
  WPERR_SERVICE          = $80004000;      // Service related error
  WPERR_SERVICEUNKNOWN   = WPERR_SERVICE+1;// Service not found
  // Link related Error Codes
  WPERR_UPLINK           = $80005000;
  WPERR_NOUPLINK         = WPERR_UPLINK+1; // No Uplink assigned
  WPERR_UPLINKUNKNOWN    = WPERR_UPLINK+2; // Unknown Uplink Name

  {============================================================================}
  {= WinPoint defined constants ===============================================}
  {============================================================================}

  {--- Area Types -------------------------------------------------------------}
  AT_Echo       = 0;               // Normal Fido-Style Echo areas with Uplink
  AT_Bad        = 1;               // BAD area
  AT_Dupes      = 2;               // DUPES area
  AT_Net        = 3;               // NETMAIL area
  AT_Merlin     = 4;               // Knows all da tricks ;-) - dont use
  AT_Separator  = 5;               // Separator in arealist
  AT_Report     = 6;               // REPORT area
  AT_Prototype  = 7;               // PROTOTYPE area (hast no data,only settings)
  AT_Internal   = 8;               // Internal Fidostyle storage areas with no uplink
  AT_EMail      = 9;               // Predefined - Dont use
  AT_Newsgroup  = 10;              // Predefined - Dont use
  AT_MSReserved = 11;              // Dont use
  AT_General    = 12;              // Non-Fidostyle generic area
  // Free IDs for Plugins etc. WP handles areas of those Types like AT_Internal
  // > Please dont use them till WP PLugin API provides allocation functions
  // > for Custom Area IDs since otherwise two plugins may collide in their ID
  // > usage!
  AT_UserFirst = 128;
  AT_UserLast  = 255;

  // Area type sets
  ATS_Fidostyle   = [AT_BAD,AT_DUPES,AT_ECHO,AT_NET,AT_INTERNAL]; // Areas that use fidostyle addressing
  ATS_Internet    = [AT_EMail,AT_Newsgroup];
  ATS_Conferences = [AT_Echo,AT_Newsgroup];                       // Conference Areas

  {--- WinPoint Message Flags -------------------------------------------------}
  MFLG_Hold     = 2;
  MFLG_Unread   = 4;
  MFLG_ToSysop  = 16;
  MFLG_Kill     = 32;
  MFLG_Send     = 64;
  MFLG_Sent     = 128;
  MFLG_Crash    = 256;
  MFLG_Attach   = 512;            // not sure if this works
  MFLG_Request  = 1024;
  MFLG_Created  = 2048;
  MFLG_Dead     = 4096;           // mail is tagged for deletion - internal use!
  MFLG_Imported = 8192;
  // Combinations
  MFLG_UnreadToSysop = MFLG_ToSysop + MFLG_Unread;
  MFLG_UnreadImported = MFLG_Imported + MFLG_Unread;
  MFLG_UnreadToSysopImported = MFLG_Imported + MFLG_Unread + MFLG_ToSysop;


  {============================================================================}
  {= FTSC defined constants ===================================================}
  {============================================================================}

  // Bit Values for TPakdMesssage.attribute
  Attr_Private              = 1;
  Attr_Crash                = 2;
  Attr_Recd                 = 4;
  Attr_Sent                 = 8;
  Attr_FileAttached         = 16;
  Attr_InTransit            = 32;
  Attr_Orphan               = 64;
  Attr_KillSent             = 128;
  Attr_Local                = 256;
  Attr_HoldForPickup        = 512;
  Attr_unused               = 1024;
  Attr_FileRequest          = 2048;
  Attr_ReturnReceiptRequest = 4096;
  Attr_IsReturnReceipt      = 8192;
  Attr_AuditRequest         = 16384;
  Attr_FileUpdateReq        = 32768;


type
   TUTCTime  = DWord;     // Seconds since 1.1.1970 00:00:00
   TThreadID = Integer;

  {============================================================================}
  {= Fidonet defined records ==================================================}
  {============================================================================}

  // Format of Type 2 stored Message as described in FTS-0001
  PPakdMessage = ^TPakdMessage;
  TPakdMessage = packed record
    messagetype  :word;
    orignode     :word;
    destnode     :word;
    orignet      :word;
    destnet      :word;
    attribute    :word;    // possible values are described above
    cost         :word;
    datetime     :array [1..20] of byte;
    {tousername   :null terminated, up to 36}
    {fromusername :null terminated, up to 36}
    {subject      :null terminated, up to 72}
  end;

  {============================================================================}
  {= WinPoint defined types ===================================================}
  {============================================================================}
  // Service Handle
  HService = Integer;

  // Service classes
  ServiceClasses = (SC_General,SC_Area,SC_Msg);

  // This record describes a plugin provided user accessible function
  PPluginService = ^TPluginService;
  TPluginService = packed record
    ServiceClass :Integer;        // a member of the ServiceClasses enumeration
    Title        :ShortString;    // Menu Title for this service
    Description  :ShortString;    // Service description
    Command      :Integer;        // Command Message ID to be sent to Plugin
    Plugin       :Integer;        // ThreadID of Plugin API
    Reserved     :Pointer;        // Reserved for future use
  end;

  // WinPoint stores all additional information about a message in this record
  PMessageInfo = ^TMessageInfo;
  TMessageInfo = packed record
    Custom        :Pointer;        {  4} {ID when stored}
    OrigAddress   :TPackedAddress; {  8} // Fidonet Address of Sender
    DestAddress   :TPackedAddress; {  8} // Fidonet Address of Receiver
    TheMSGIDCRC   :DWord;          {  4} // CRC32 over the Original Mails MSGID
    TheREPLYCRC   :DWord;          {  4} // CRC32 over the Original Mails REPLY
    OrigName      :string[35];     { 36}
    DestName      :string[35];     { 36}
    Subject       :string[71];     { 72}
    Area          :string[51];     { 52}
    MsgDate       :TUTCTime;       {  4} // Date in the Message
    TossDate      :TUTCTime;       {  4} // Date we received that mail
    UserData      :DWord;          {  4} {at runtime!}
    TheMSGID      :DWord;          {  4}
    TheREPLY      :DWord;          {  4}
    MsgHeaderSize :Word;           {  2}
    MsgSize       :DWord;          {  4}
                                   {---}
                                   {250 Bytes}
  end;

  // This record is used to exchange program descriptions via GETINFO
  PAppInfo = ^TAppInfo;
  TAppInfo = packed record
    Name        :ShortString;    // Name of the program, i.e. 'SuperEdit'
    Copyright   :ShortString;    // Copyright note
    Description :ShortString;    // Description, i.e. 'Alternative Mail Editor';
    Version     :Integer;        // Internal program version
    Revision    :Integer;        // Internal program revision
    APIReqd     :Integer;        // Minimum API Version requiered
    APISupp     :Integer;        // Maximum API Version used (may be higher !!!)
  end;

  // WinPoint uses this record to store all administrative information for a msg
  PStoredHeader = ^TStoredHeader;
  TStoredHeader = packed record
    MessageInfo   :TMessageInfo;   // 250
    Index         :Longint;        //   4
    Flags         :Word;           //   2 -> 256 Bytes
  end;

  // This record is used by WinPoint to create a new message
  PNewMail = ^TNewMail;
  TNewMail = packed record
    DestName      :string[35];     // 36
    OrigName      :string[35];     // 36
    Subject       :string[71];     // 72
    Origin        :string[71];     // 72
    DestAddress   :TPackedAddress; //  8
    OrigAddress   :TPackedAddress; //  8
    Flags         :Word;           //  2
    Size          :DWord           //  4 -> 218 Bytes
    {Text         :null terminated text}
  end;

  // This record is used by WinPoint to create a new area
  PNewArea = ^TNewArea;
  TNewArea = packed record
    Name     :ShortString;         // Name for new Echo
    Uplink   :ShortString;         // if AreaType = AT_ECHO
    AreaType :Byte;                // Type
  end;

  // This record represents Fidonet Uplink parameters
  PNetLinkInfo = ^TNetLinkInfo;
  TNetLinkInfo = packed record
    UplinkName    :ShortString;
    SysopName     :ShortString;
    Uplinkaddress :TPackedAddress;
    Sysopaddress  :TPackedAddress;
  end;

  {============================================================================}
  {= Message types ============================================================}
  {============================================================================}
  // this message is sent to WinPoint to get the API connection
  TWPMQueryAPI = record
    Msg               :Cardinal;  // WP_QUERY_API
    PluginThreadID    :Longint;   // Thread Identifier of Plugin Interface
    ClientThreadID    :Longint;   // Thread Identifier of Client
    case Integer of
    0:(ServerThreadID :Longint);  // Thread Identifier of Server Connection
    1:(Result         :Longint);  // Thread Identifier of Server Connection
  end;

  {============================================================================}
  {= Support Classes adapted from Borlands IPC Demo ===========================}
  {============================================================================}
  THandledObject = class(TObject)
  protected
    FHandle: THandle;
  public
    destructor Destroy; override;
    property Handle: THandle read FHandle;
  end;
  TSharedMem = class(THandledObject)
  private
    FName: string;
    FSize: Integer;
    FCreated: Boolean;
    FFileView: Pointer;
  public
    constructor Create(const Name: string; Size: Integer);
    destructor Destroy; override;
    property Name: string read FName;
    property Size: Integer read FSize;
    property Buffer: Pointer read FFileView;
    property Created: Boolean read FCreated;
  end;


function SendThreadMessage(ThreadID:TThreadID; Msg:UINT; wParam:WPARAM;
                           lParam:LPARAM; var Res:DWORD):Boolean;
function GetAppInfo(Server:TThreadID; var AppInfo:TAppInfo):Boolean;

implementation

function SendThreadMessage(ThreadID:TThreadID; Msg:UINT; wParam:WPARAM;
                           lParam:LPARAM; var Res:DWORD):Boolean;
// Sends a message to the given thread and waits for a WP_RESULT message
var
  RepMsg:TMsg;
begin
     if PostThreadMessage(ThreadID,Msg,wParam,lParam) then
     begin
          Result:=GetMessage(RepMsg,0,WP_RESULT,WP_RESULT);
          Res   :=RepMsg.LParam;
     end else Result:=False;
end;

function GetAppInfo(Server:TThreadID; var AppInfo:TAppInfo):Boolean;
// Get the Process info
// Note: the Server is in this case the asked application
{Exceptions: }
var
  Client :TThreadID;
  Res    :DWORD;
  Buffer :TSharedMem;
  Size   :Integer;
begin
     Client:=GetCurrentThreadID;
     if SendThreadMessage(Server,WP_GETINFO,0,Client,Res) then
     begin
          if Res=WP_SUCCESS then
          begin
               // Handle Result
               Size   :=SizeOf(TAPPInfo);
               Buffer :=TSharedMem.Create('WPPluginInfo'+IntToHex(Server,8),
                                          Size);
               if (not Buffer.Created) and (Buffer.Buffer<>nil)
               then Move(Buffer.Buffer^,AppInfo,Size);
               Buffer.Destroy;
               // Acknowledge Message
               PostThreadMessage(Server,WP_RESULT,0,0);
               Result:=True;
          end else Result:=False;
     end else Result:=False;
end;

{ THandledObject }

destructor THandledObject.Destroy;
begin
  if FHandle <> 0 then
    CloseHandle(FHandle);
end;

{ TSharedMem }

constructor TSharedMem.Create(const Name: string; Size: Integer);
begin
     FName     := Name;
     FSize     := Size;
     FFileView :=nil;
     { CreateFileMapping, when called with $FFFFFFFF for the hanlde value,
       creates a region of shared memory }
     FHandle := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0,
         Size, PChar(Name));
     if FHandle<>0 then
     begin
          FCreated := GetLastError = 0;
          { We still need to map a pointer to the handle of the shared memory region }
          FFileView := MapViewOfFile(FHandle, FILE_MAP_WRITE, 0, 0, Size);
     end;
end;

destructor TSharedMem.Destroy;
begin
     if FFileView<>nil
     then UnmapViewOfFile(FFileView);
     inherited Destroy;
end;

end.
