//==============================================================================
// WinPoint 95 API wrapper classes for Borland Delphi 2.0, 3.0 and 4.0
//==============================================================================
//
// (C) 1998-1999 by Tim Schattkowsky
//
// API Version implemented: 9
// Last Revision: 28.04.1998
//
// NOTE: WinPoint 95 Release 1.6A or later is requiered !
//==============================================================================
// Changes
//==============================================================================
// 07.05.1998  Added Disconnecting() and Disconnected() event wrapper methods
//             These methods now also prevent OnDisconnecting and OnDisconnected
//             Events from occuring when the component is destroying. This
//             caused problems in programs that do not manually disconnect and
//             have handlers for one of these events that use other objects
//             that area possibly already destroyed.
//             WARNING: Now Your handlers for OnDisconnected and OnDisconnecting
//                      will not be called if TWinPointAPI has to disconnect
//                      because it gets destroyed. To prevent this You may have
//                      to explicitly Disconnect() before the TWinPointAPI gets
//                      destroyed. Usually this is the case when the plugin
//                      quits, so You may handle this situation in your exit
//                      code, for example in OnFormClose for the main form.
// 07.05.1998  Added CreateArea() method to create new areas in WinPoint
// 19.05.1998  TWPAreaService now returns a list of Areas instead of one.
//             New TWPMsgService Class. Note that a SC_Msg Message implicates
//             an area change in the host. Use the new Function GetCurrentArea()
//             to determine the currently active area then.
// 22.05.1998  All Error Code are now negative so theres no more mess with sign.
//             New function GetAreaUplink and Exception ENoUplink.
//             Some Expetions were not raised
// 23.05.1998  New TNetLinkList support class.
//             New GetUplinks() function.
// 25.05.1998  New GetAreaType() function
// 04.06.1998  GetAreaUplink handled wrong error codes
//             TMessageList und TNetLinkList now correctly call ClearContent in
//             their destructors
// 02.07.1998  New TWPMsg class to simplify message reading and writing.
//             GetMsg is now declared as function.
// 12.09.1998  ServerDisconnected will no longer  be fired if in csDestroying.
// 12.09.1998  TPluginAPI not posts all events to the main thread instead of
//             using synchronize to process it. This is very important since
//             the other way plugins that terminated in OnServerDisconnected
//             or from an command handler had a chance of crashing the windows
//             kernel by terminating the thread in a system function
//             (SendMessage from Synchronize). Terminating a plugin from a
//             command handler is not common anyway, but from
//             OnServerDisconnected this is a quite usual behavior. So I believe
//             this was the problem that caused the heavy crashes on some
//             windows 9x machines when running the sample plugins. The problem
//             occured only on some machines and most likely not under NT.
//             Please tell me if the new code still produces it!
// 01.09.1998  TWinPointAPI did not wait for the TPluginAPI thread to terminate.
//             This caused Exceptions on some machines under Win9x when the
//             exited in an OnServerDisconnected event. The problem should
//             hopefully ge gone now. Tell me if not. At least on the test
//             machine I finally encountered it works fine now!
// 22.04.1998  API version 9 now allows access to WinPoints inbuilt dial
//             translations.
// 28.04.1998  Fixed lots of type mismatches for Delphi 4.0
//             Fixed a problem that caused Delphi to crash when the package got
//             removed
//==============================================================================
unit WinpointAPI;

interface

uses
  SysUtils, Classes, Windows, Messages, WPAPICommon, Forms, Dialogs, FidoAddr;

const
  CurrentAPIVersion = 9;             // Version these classes area designed for

type
  // Forward declarations
  TWinPointAPI     = class;
  TCustomWPService = class;

  // Exceptions
  EAPIError=class(Exception);        // all Error Exceptions inherited from this
  EUnknownAPIError=class(EAPIError); // We just dont know what went wrong
  EWrongFormat=class(EAPIError);     // Data Format wrong
  // Generic Exceptions
  ECmdFailed=class(EAPIError);
  ENotConnected=class(EAPIError);    // No connection to WinPoint established
  // Area related Exceptions
  EAreaInUse=class(EAPIError);
  EAreaNotOpen=class(EAPIError);     // Operation requiered open area but ...
  EAreaOpenFailed=class(EAPIError);
  EAreaUnknown=class(EAPIError);     // Area does not exist in WinPoint
  EAreaExists=class(EAPIError);      // Area already exists in WinPoint
  // Service related Exceptions
  EServiceUnknown=class(EAPIError);
  // Message related Exceptions
  EMsgBadIndex=class(EAPIError);
  // Uplink related Exceptions
  EUplinkUnknown=class(EAPIError);
  ENoUplink=class(EAPIError);
  {============================================================================}
  {=== TWPMsg =================================================================}
  {============================================================================}
  TWPMsg = class
  private
    // property variables
    FArea       :String;
    FDestAddr   :TFidonetAddress;
    FDestName   :String;
    FFlags      :Word;
    FLines      :TStringList;
    FOrigAddr   :TFidonetAddress;
    FOrigin     :String;
    FOrigName   :String;
    FSubject    :String;
    // Property acess
    function    GetLines:TStrings;
    procedure   SetDestAddr(AAddr:TFidonetAddress);
    procedure   SetLines(NewLines:TStrings);
    procedure   SetOrigAddr(AAddr:TFidonetAddress);
    //
    procedure   CreateMe;
  public
    constructor Create;
    constructor CreateFromMsg(Msg:PPakdMessage);
    constructor CreateFromMsgAndFree(Msg:PPakdMessage);
    destructor  Destroy; override;
    // Methods
    procedure   AssignPakdMsg(Msg:PPakdMessage);
    function    CreateNewMail:PNewMail;
    // Properties
    property    Area:String read FArea write FArea;
    property    DestAddress:TFidonetAddress read FDestAddr write SetDestAddr;
    property    DestName:String read FDestName write FDestName;
    property    Flags:Word read FFlags write FFlags;
    property    Lines:TStrings read GetLines write SetLines;
    property    OrigAddress:TFidonetAddress read FOrigAddr write SetOrigAddr;
    property    Origin:String read FOrigin write FOrigin;
    property    OrigName:String read FOrigName write FOrigName;
    property    Subject:String read FSubject write FSubject;
  end;


  {============================================================================}
  {=== TServiceList ===========================================================}
  {============================================================================}
  TServiceList = class(TList)
  private
    function  RItems(Index:Integer):TCustomWPService;
    procedure WItems(Index:Integer; Item:TCustomWPService);
  public
    // inherited methods
    function  Add(Item: TCustomWPService): Integer;
    function  First: TCustomWPService;
    function  IndexOf(Item: TCustomWPService): Integer;
    procedure Insert(Index: Integer; Item: TCustomWPService);
    function  Last: TCustomWPService;
    function  Remove(Item: TCustomWPService): Integer;
    // Properties
    property  Items[Index: Integer]: TCustomWPService
              read RItems write WItems; default;
  end;


  {============================================================================}
  {=== TNetLinkList ===========================================================}
  {============================================================================}
  TNetLinkList = class(TList)
  private
    function   RItems(Index:Integer):PNetLinkInfo;
    procedure  WItems(Index:Integer; Item:PNetLinkInfo);
  public
    destructor Destroy; override;
    // inherited methods
    function   Add(Item: PNetLinkInfo): Integer;
    function   First: PNetLinkInfo;
    function   IndexOf(Item: PNetLinkInfo): Integer;
    procedure  Insert(Index: Integer; Item: PNetLinkInfo);
    function   Last: PNetLinkInfo;
    function   Remove(Item: PNetLinkInfo): Integer;
    // New Methods
    procedure  ClearContent;
    // Properties
    property   Items[Index: Integer]: PNetLinkInfo
               read RItems write WItems; default;
  end;

  {============================================================================}
  {=== TMessagelist ===========================================================}
  {============================================================================}
  TMessageList = class(TList)
  private
    function  RItems(Index:Integer):PStoredHeader;
    procedure WItems(Index:Integer; Item:PStoredHeader);
  public
    destructor Destroy; override;
    // inherited methods
    function  Add(Item: PStoredHeader): Integer;
    function  First: PStoredHeader;
    function  IndexOf(Item: PStoredHeader): Integer;
    procedure Insert(Index: Integer; Item: PStoredHeader);
    function  Last: PStoredHeader;
    function  Remove(Item: PStoredHeader): Integer;
    // new methods
    procedure ClearContent;   // Dispose content and clear list
    // Properties
    property  Items[Index: Integer]:PStoredHeader
              read RItems write WItems; default;
  end;

  {============================================================================}
  {=== TPluginAPI =============================================================}
  {============================================================================}
  TPluginAPI = class(TThread)
  private
    API         :TWinPointAPI;
    Ready       :DWORD;
    FThreadID   :DWORD;
    Msg         :TMsg;
    //
    procedure   ReturnInfo(Receiver:Integer);
  protected
    procedure   Execute; override;
    procedure   DoTerminate; override;
  public
    constructor Create(WPAPI:TWinPointAPI; var Success:Boolean);
    property    ThreadID:DWORD read FThreadID;
  end;


  {============================================================================}
  {=== TCustomWPService =======================================================}
  {============================================================================}
  TPluginCmdEvent = procedure (Msg:TMsg) of object;
  TCustomWPService = class(TComponent)
  private
    // Private Variables
    Ready       :Boolean;
    // Property Variables
    FActive     :Boolean;
    FAPI        :TWinPointAPI;
    FClass      :ServiceClasses;
    FCMD        :DWORD;
    FDesc       :ShortString;
    FOnCmd      :TPluginCmdEvent;
    FHandle     :HService;
    FTitle      :ShortString;
    // private methods
    procedure   SetActive(State:Boolean);
    procedure   SetAPI(NewAPI:TWinPointAPI);
    procedure   SetCMD(NewCMD:DWORD);
  protected
    procedure   Loaded; override;
    procedure   Notification(AComponent:TComponent; Operation:TOperation); override;
    procedure   DoCommand(Msg:TMsg); virtual;
    // Properties
    property    Active:Boolean              read FActive write SetActive;
    property    Command:DWORD               read FCMD write SetCMD;
    property    Description:ShortString     read FDesc write FDesc;
    property    Handle:HService             read FHandle write FHandle;
    property    ServiceClass:ServiceClasses read FClass write FClass;
    property    Title:ShortString           read FTitle write FTitle;
    property    WinPointAPI:TWinPointAPI    read FAPI write SetAPI;
    // Events
    property    OnCommand:TPluginCmdEvent   read FOnCmd write FOnCmd;
  public
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
  end;
  TWinPointService = class(TCustomWPService)
  // SC_General WinPoint API Service
  published
    // Properties
    property    Active;
    property    Command;
    property    Description;
    property    WinPointAPI;
    property    Title;
    // Events
    property    OnCommand;
  end;
  TAreaPluginCmdEvent = procedure (Areas:TStringList; Msg:TMsg) of object;
  TAreaService = class(TCustomWPService)
  // SC_Area WinPoint API Service
  private
    FOnCommand :TAreaPluginCmdEvent;
  protected
    procedure   DoCommand(Msg:TMsg); override;
  public
    constructor Create(AOwner:TComponent); override;
  published
    // Properties
    property    Active;
    property    Command;
    property    Description;
    property    WinPointAPI;
    property    Title;
    // Events
    property    OnCommand:TAreaPluginCmdEvent read FOnCommand write FOnCommand;
  end;
  TMsgPluginCmdEvent = procedure (Msgs:TList; Msg:TMsg) of object;
  TMsgService = class(TCustomWPService)
  // SC_Msg WinPoint API Service
  private
    FOnCommand :TMsgPluginCmdEvent;
  protected
    procedure   DoCommand(Msg:TMsg); override;
  public
    constructor Create(AOwner:TComponent); override;
  published
    // Properties
    property    Active;
    property    Command;
    property    Description;
    property    WinPointAPI;
    property    Title;
    // Events
    property    OnCommand:TMsgPluginCmdEvent read FOnCommand write FOnCommand;
  end;


  {============================================================================}
  {=== TWinPointAPI ===========================================================}
  {============================================================================}
  TWinPointAPI = class(TComponent)
  private
    // Instance Variables
    API         :Integer;          // WinPoint API thread ID
    PluginAPI   :TPluginAPI;
    Services    :TServiceList;
    // Property Variables
    FAPIVersion :Integer;
    FAutoConn   :Boolean;
    FConnected  :Boolean;
    FOnCmd      :TPluginCmdEvent;
    FOnConn     :TNotifyEvent;
    FOnConned   :TNotifyEvent;
    FOnConnF    :TNotifyEvent;
    FOnDisc     :TNotifyEvent;
    FOnDisced   :TNotifyEvent;
    FOnSDisconn :TNotifyEvent;
    FOnWrong    :TNotifyEvent;
    // Property access handler
    procedure   SetConnected(NewState:Boolean);
    // private methods
    procedure   HandleCommand(Msg:TMsg);
  protected
    APPInfo     :TAppInfo;
    // overriden methods
    procedure   Loaded; override;
    // protected methods
    procedure   Acknowledge;
    function    SendStringCommand(Cmd:Cardinal; ID,Data:String;
                                  var Res:DWORD):Boolean;
    // Event wrapper
    procedure   ServerDisconnected; virtual;
    procedure   Disconnected; virtual;
    procedure   Disconnecting; virtual;
    // Callbacks
    class procedure HandleMsgs(var Msg: TMsg; var Handled: Boolean);
  public
    constructor Create(AOwner:TComponent); override;
    destructor  Destroy; override;
    // Manage
    procedure   DeRegisterManagedService(Service:TCustomWPService);
    procedure   DontManage(Service:TCustomWPService);
    procedure   Manage(Service:TCustomWPService);
    procedure   RegisterManagedService(Service:TCustomWPService);
    // WinPoint 95 API Functions
    procedure   CloseArea;
    procedure   Connect;
    procedure   CreateArea(AName,AUplink:String; AType:Byte);
    procedure   DeRegisterService(Service:HService);
    procedure   Disconnect;
    procedure   GetAreaList(AreaList:TStrings);
    function    GetAreaType(Name:String):DWORD;
    procedure   GetAreaUplink(Name:String; var Uplink:TNetLinkInfo);
    function    GetCurrentArea:String;
    procedure   GetDialTranslations(Translations:TStrings);
    procedure   GetInfo(var AppInfo:TAppInfo);
    function    GetMsg(Index:Integer):PPakdMessage;
    function    GetMsgFlags(Index:Integer):Word;
    procedure   GetMsgList(MsgList:TMessagelist);
    procedure   GetUplinks(Uplinks:TNetLinkList);
    procedure   OpenArea(Name:String);
    function    RegisterService(Service:TPluginService):HService;
    procedure   SetDialTranslations(Translations:TStrings);
    procedure   SetMsgFlags(Index:Integer; NewFlags:Word);
    procedure   StoreMsg(NewMail:PNewMail);
    // Properties
    property    APIVersion:Integer
                read  FAPIVersion;
    property    Connected:Boolean
                read  FConnected
                write SetConnected;
  published
    property    AutoConnect:Boolean
                read  FAutoConn
                write FAutoConn
                default True;
    property    PICopyright:ShortString
                read  APPInfo.Copyright
                write APPInfo.Copyright;
    property    PIDescription:ShortString
                read  APPInfo.Description
                write APPInfo.Description;
    property    PIName:ShortString
                read  APPInfo.Name
                write APPInfo.Name;
    property    PIAPIRequiered:Integer
                read  APPInfo.APIReqd
                write APPInfo.APIReqd;
    property    PIRevision:Integer
                read  APPInfo.Revision
                write APPInfo.Revision;
    property    PIAPISupported:Integer
                read  APPInfo.APISupp
                write APPInfo.APISupp;
    property    PIVersion:Integer
                read  APPInfo.Version
                write APPInfo.Version;
    // Events
    property    OnCommand:TPluginCmdEvent
                read  FOnCmd
                write FOnCmd;
    property    OnConnected:TNotifyEvent
                read  FOnConn
                write FOnConn;
    property    OnConnectFailed:TNotifyEvent
                read  FOnConnF
                write FOnConnF;
    property    OnConnecting:TNotifyEvent
                read  FOnConned
                write FOnConned;
    property    OnDisconnected:TNotifyEvent
                read  FOnDisced
                write FOnDisced;
    property    OnDisconnecting:TNotifyEvent
                read  FOnDisc
                write FOnDisc;
    property    OnServerDisconnected:TNotifyEvent
                read  FOnSDisconn
                write FOnSDisconn;
    property    OnWrongVersion:TNotifyEvent
                read  FOnWrong
                write FOnWrong;
  end;


procedure Register;

implementation
{$R WinPointAPI.res}

var           
  OldOnMessage :TMessageEvent;

function FindChar(AChar:Char; TXT:PChar; Limit:Integer):Integer;
var Data :Pointer;
    Res  :Pointer;
    Find :ANSIChar;
    Lim  :Integer;
label Scan,Found,Done,NoFO;
begin
     if (Txt=nil) or (Txt^=#0) then
     begin
          FindChar:=0;
          Exit;
     end;
     Data :=Txt;
     Res  :=@Result;
     Find :=AChar;
     Lim  :=Limit;
     asm
        push   esi
        push   edi
        push   ebx
        mov    esi,Data
        mov    bl,Find
        mov    edx,esi
        mov    ecx,Lim
Scan:   mov    al,[esi]
        inc    esi
        cmp    al,bl
        je     Found
        dec    ecx
        jz     NoFo
        cmp    al,0
        jne    Scan
NoFo:   {Not Found}
        mov    edi,Res
        mov    dword ptr [edi],0
        jmp    Done
Found:  {Yeah, ist there!}
        mov    edi,Res
        sub    esi,edx
        mov    [edi],esi
Done:   {Deinit ASM}
        pop    ebx
        pop    edi
        pop    esi
     end;
end;

function GetNextString(var TXT:String):string;
var
  Index :Integer;
  Size  :Integer;
  Max   :Integer;
begin
     Index  :=1;
     Size   :=0;
     Max    :=Length(TXT);
     while  (Index<=Max)      and (TXT[Index]<=' ')     do Inc(Index);
     while  (Index+Size<=Max) and (TXT[Index+Size]>' ') do Inc (Size);
     Result:=Copy(TXT,Index,Size);
     Delete(TXT,1,Index+Size-1);
end;

function ToANSI(AString:String):String;
begin
     Result:=AString;
     OEMToChar(@Result[1],@Result[1]);
end;

{==============================================================================}
{=== TWPMsg implementation ====================================================}
{==============================================================================}
constructor TWPMsg.Create;
begin
     inherited Create;
     CreateMe;
end;

constructor TWPMsg.CreateFromMsg(Msg:PPakdMessage);
begin
     inherited Create;
     CreateMe;
     AssignPakdMsg(Msg);
end;

constructor TWPMsg.CreateFromMsgAndFree(Msg:PPakdMessage);
begin
     inherited Create;
     CreateMe;
     if Assigned(Msg) then
     begin
          AssignPakdMsg(Msg);
          FreeMem(Msg);
     end;
end;

function TWPMsg.CreateNewMail:PNewMail;
var
  Ts      :String;
  i,MSize :Integer;
begin
     Ts     :='';
     for i:=0 to Lines.Count-1 do Ts:=Ts+Lines[i]+#$0d#$0a;
     MSize  :=SizeOf(TNewMail)+Length(Ts)+1;
     GetMem(Result,MSize);
     with Result^ do
     begin
          DestName    :=Self.Destname;
          OrigName    :=Self.OrigName;
          Subject     :=Self.Subject;
          Origin      :=Self.Origin;
          DestAddress :=Self.DestAddress.Address;
          OrigAddress :=Self.OrigAddress.Address;
          Flags       :=Self.Flags;
          Size        :=MSize;
          Move(Ts[1],Pointer(Integer(Result)+SizeOf(TNewMail))^,Length(Ts)+1);
     end;
end;

procedure TWPMsg.AssignPakdMsg(Msg:PPakdMessage);
var
  ts:String;
  Bdy:PChar;
  CRIndex:Integer;
  Start,Temp, BO, BC :Integer;
begin
     if Msg<>nil then
     begin
          // Setup Header
          Bdy            :=Pointer(Integer(Msg)+SizeOf(TPakdMessage));
          FDestName      :=Bdy;
          Bdy            :=Bdy+Length(FDestName)+1;
          FOrigName      :=Bdy;
          Bdy            :=Bdy+Length(FOrigName)+1;
          FSubject       :=Bdy;
          Bdy            :=Bdy+Length(FSubject)+1;
          FOrigAddr.Text :='0:0/0';
          FDestAddr.Text :='0:0/0';
          FFlags         :=0;
          FArea          :='NETMAIL';  // might be taken from Mail
          FOrigin        :='';
          // Scan Raw Mail
          repeat
                if Bdy^=#10 then Inc(Bdy);
                {so handle that line!}
                CRIndex:=FindChar(#$0d,Bdy,-1);
                // Handle Line
                if (Copy(Bdy,1,1)<>#1) then
                begin
                     if CRIndex>0 then Bdy[CRIndex-1]:=#0;
                     if Copy(Bdy,1,5)='AREA:' then
                     begin
                          // Handle AREA:
                          Inc(Integer(Bdy),5);
                          Dec(CRIndex,5);
                          Ts:=Bdy;
                          FArea:=GetNextString(ts);
                     end else
                     if Copy(Bdy,1,11) = ' * Origin: ' then
                     begin
                          // Handle Origin
                          ts :=' * Origin: ';
                          Inc(Integer(Bdy),Length(ts));
                          Dec(CRIndex,Length(ts));
                          FOrigin:=PChar(Bdy);
                          // Fix Origin
                          Start :=Length(FOrigin);
                          Temp  :=Start;
                          while (Temp>0) and (FOrigin[Temp]<>')') do Dec(Temp);
                          BC :=Temp;
                          while (Temp>0) and (FOrigin[Temp]<>'(') do Dec(Temp);
                          BO :=Temp;
                          Dec(Temp);
                          while (Temp>0) and (FOrigin[Temp]<=' ') do Dec(Temp);
                          // Get Orig address
                          FOrigAddr.Text :=Copy(FOrigin,BO+1,BC-BO-1);
                          Delete(FOrigin,Temp+1,Start-Temp);
                     end else
                     if Copy(Bdy,1,4) = '--- ' then
                     begin
                          // Handle Tearline
                          // FTearline :=PChar(Bdy); just ignoring it!
                     end else
                     begin
                          // Add this line!
                          if CRIndex>0
                          then FLines.Add(ToANSI(Bdy))
                          else if Length(Bdy)>0 then FLines.Add(ToANSI(Bdy));
                     end;
                end;
                if CRIndex>0 then Bdy[CRIndex-1]:=#$0d;
                if CRIndex>0 then Bdy :=Pointer(LongInt(Bdy)+CRIndex);
          until CRIndex=0;
     end;
end;

destructor TWPMsg.Destroy;
begin
     FLines.Destroy;
     FDestAddr.Destroy;
     FOrigAddr.Destroy;
     inherited Destroy;
end;

procedure TWPMsg.CreateMe;
begin
     FLines    :=TStringList.Create;
     FDestAddr :=TFidonetAddress.Create;
     FOrigAddr :=TFidonetAddress.Create;
end;

function TWPMsg.GetLines:TStrings;
begin
     Result:=FLines;
end;

procedure TWPMsg.SetLines(NewLines:TStrings);
begin
     FLines.Assign(NewLines);
end;

procedure TWPMsg.SetDestAddr(AAddr:TFidonetAddress);
begin
     FDestAddr.Assign(AAddr);
end;

procedure TWPMsg.SetOrigAddr(AAddr:TFidonetAddress);
begin
     FOrigAddr.Assign(AAddr);
end;

{==============================================================================}
{=== TServiceList implementation ==============================================}
{==============================================================================}

{--- inherited methods modifications to use PStoredHeader instead of Pointers -}

function  TServiceList.Add(Item: TCustomWPService): Integer;
begin
     Result := inherited Add(Item);
end;

function  TServiceList.First: TCustomWPService;
begin
     Result := inherited First;
end;

function  TServiceList.IndexOf(Item: TCustomWPService): Integer;
begin
     Result := inherited IndexOf(Item);
end;

procedure TServiceList.Insert(Index: Integer; Item: TCustomWPService);
begin
     inherited Insert(Index,Item);
end;

function  TServiceList.Last: TCustomWPService;
begin
     Result := inherited Last;
end;

function  TServiceList.Remove(Item: TCustomWPService): Integer;
begin
     Result := inherited Remove(Item);
end;

{--- New methods --------------------------------------------------------------}

function  TServiceList.RItems(Index:Integer):TCustomWPService;
begin
     Result := inherited Items[Index];
end;

procedure TServiceList.WItems(Index:Integer; Item:TCustomWPService);
begin
     inherited Items[Index] := Item;
end;



{==============================================================================}
{=== TNetLinkList implementation ==============================================}
{==============================================================================}

{--- inherited methods modifications to use PStoredHeader instead of Pointers -}

destructor TNetLinkList.Destroy;
begin
     ClearContent;
     inherited Destroy;
end;

function  TNetLinkList.Add(Item: PNetLinkInfo): Integer;
begin
     Result := inherited Add(Item);
end;

function  TNetLinkList.First: PNetLinkInfo;
begin
     Result := inherited First;
end;

function  TNetLinkList.IndexOf(Item: PNetLinkInfo): Integer;
begin
     Result := inherited IndexOf(Item);
end;

procedure TNetLinkList.Insert(Index: Integer; Item: PNetLinkInfo);
begin
     inherited Insert(Index,Item);
end;

function  TNetLinkList.Last: PNetLinkInfo;
begin
     Result := inherited Last;
end;

function  TNetLinkList.Remove(Item: PNetLinkInfo): Integer;
begin
     Result := inherited Remove(Item);
end;

{--- New methods --------------------------------------------------------------}

function  TNetLinkList.RItems(Index:Integer):PNetLinkInfo;
begin
     Result := inherited Items[Index];
end;

procedure TNetLinkList.WItems(Index:Integer; Item:PNetLinkInfo);
begin
     inherited Items[Index] := Item;
end;

procedure TNetLinkList.ClearContent;
{ This method first disposes all the TNetLinkInfo records the list holds
  pointers to. This is useful since TWinPointAPI.GetUplinks allocates
  all nodes in the list it returns and the user has to free them. Finally
  the list will be Clear()ed.                                              }
var
  i:Integer;
begin
     for i:=0 to Count-1 do Dispose(Items[i]);
     Clear;
end;


{==============================================================================}
{=== TMessageList implementation ==============================================}
{==============================================================================}

{--- inherited methods modifications to use PStoredHeader instead of Pointers -}

destructor TMessagelist.Destroy;
begin
     ClearContent;
     inherited Destroy;
end;

function  TMessagelist.Add(Item: PStoredHeader): Integer;
begin
     Result := inherited Add(Item);
end;

function  TMessagelist.First: PStoredHeader;
begin
     Result := inherited First;
end;

function  TMessagelist.IndexOf(Item: PStoredHeader): Integer;
begin
     Result := inherited IndexOf(Item);
end;

procedure TMessagelist.Insert(Index: Integer; Item: PStoredHeader);
begin
     inherited Insert(Index,Item);
end;

function  TMessagelist.Last: PStoredHeader;
begin
     Result := inherited Last;
end;

function  TMessagelist.Remove(Item: PStoredHeader): Integer;
begin
     Result := inherited Remove(Item);
end;

{--- New methods --------------------------------------------------------------}

function  TMessagelist.RItems(Index:Integer):PStoredHeader;
begin
     Result := inherited Items[Index];
end;

procedure TMessagelist.WItems(Index:Integer; Item:PStoredHeader);
begin
     inherited Items[Index] := Item;
end;

procedure TMessagelist.ClearContent;
{ This method first disposes all the TStoredHeader records the list holds
  pointers to. This is useful since TWinPointAPI.GetMessageList allocates
  all nodes in the list it returns and the user has to free them. Finally
  the list will be Clear()ed.                                              }
var
  i:Integer;
begin
     for i:=0 to Count-1 do Dispose(Items[i]);
     Clear;
end;



{==============================================================================}
{=== TCustomWPService implementation ==========================================}
{==============================================================================}

constructor TCustomWPService.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     FActive :=True;
     FCMD    :=WP_FIRSTSERVICE;
     FAPI    :=nil;
     FClass  :=SC_General;
     FDesc   :='No description!';
     FTitle  :='New Command';
     Ready   :=False;
end;

destructor TCustomWPService.Destroy;
begin
     WinPointAPI :=nil;
     inherited Destroy;
end;

procedure TCustomWPService.DoCommand(Msg:TMsg);
begin
     if Assigned(OnCommand) then OnCommand(Msg);
end;

procedure TCustomWPService.SetCMD(NewCMD:DWORD);
begin
     if NewCMD<WP_FIRSTSERVICE then NewCMD:=WP_FIRSTSERVICE else
     if NewCMD>WP_LASTSERVICE  then NewCMD:=WP_LASTSERVICE;
     FCMD:=NewCMD;
end;

procedure TCustomWPService.Notification(AComponent:TComponent;
                                        Operation:TOperation);
begin
     inherited Notification(AComponent,OPeration);
     if (AComponent=WinPointAPI) and (Operation=opRemove)
     then WinPointAPI:=nil;
end;

procedure TCustomWPService.Loaded;
begin
     Ready:=True;
     if FAPI<>nil then FAPI.Manage(Self);
end;

procedure TCustomWPService.SetAPI(NewAPI:TWinPointAPI);
begin
     if FAPI<>NewAPI then
     begin
          if Ready and (FAPI<>nil) then FAPI.DontManage(Self);
          FAPI :=NewAPI;
          if Ready and (FAPI<>nil) then FAPI.Manage(Self);
     end;
end;

procedure TCustomWPService.SetActive(State:Boolean);
begin
     if FActive<>State then
     begin
          if FAPI<>nil then FAPI.DontManage(Self);
          FActive:=State;
          if FAPI<>nil then FAPI.Manage(Self);
     end;
end;


{==============================================================================}
{=== TWPAreaService implementation ============================================}
{==============================================================================}

constructor TAreaService.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     FClass  :=SC_Area;
end;

procedure TAreaService.DoCommand(Msg:TMsg);
// Note: The Handler MUST Destroy Names
var
  Buffer :TSharedMem;
  Names  :TStringList;
  Cnt,i  :Integer;
  Name   :PChar;
  Temp   :String;
begin
     // Get Areaname
     Buffer :=TSharedMem.Create('WPAreaName'+IntToHex(Msg.lParam,8),Msg.wParam);
     Names  :=TStringList.Create;
     if (not Buffer.Created) and (Buffer.Buffer<>nil) then
     begin
          Cnt   :=Integer(Buffer.Buffer^);
          Name  :=Pointer(Integer(Buffer.Buffer)+4);
          for i:=1 to Cnt do
          begin
               Temp :=Name;
               Name :=Pointer(Name+Length(Temp)+1);
               Names.Add(Temp);
          end;
     end;
     Buffer.Destroy;
     // Acknowledge to WinPoint
     PostThreadMessage(Msg.lParam,WP_ACK,0,0);
     // Handle Command
     if Names.Count>0 then
     begin
          if Assigned(OnCommand) then OnCommand(Names,Msg);
     end else Names.Destroy;
end;


{==============================================================================}
{=== TWPMsgService implementation =============================================}
{==============================================================================}

constructor TMsgService.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     FClass  :=SC_Msg;
end;

procedure TMsgService.DoCommand(Msg:TMsg);
// Note: The Handler MUST Destroy Msgs
var
  Buffer :TSharedMem;
  Msgs   :TList;
  i      :Integer;
  Data   :Pointer;
begin
     // Get Areaname
     Buffer :=TSharedMem.Create('WPMsgSel'+IntToHex(Msg.lParam,8),Msg.wParam);
     Msgs   :=TList.Create;
     if (not Buffer.Created) and (Buffer.Buffer<>nil) then
     begin
          Data  :=Buffer.Buffer;
          for i:=1 to Msg.wParam shr 2 do
          begin
               Msgs.Add(Pointer(Data^));
               Data :=Pointer(Integer(Data)+4);
          end;
     end;
     Buffer.Destroy;
     // Acknowledge to WinPoint
     PostThreadMessage(Msg.lParam,WP_ACK,0,0);
     // Handle Command
     if Msgs.Count>0 then
     begin
          if Assigned(OnCommand) then OnCommand(Msgs,Msg);
     end else Msgs.Destroy;
end;


{==============================================================================}
{=== TPluginAPI implementation ================================================}
{==============================================================================}

constructor TPluginAPI.Create(WPAPI:TWinPointAPI; var Success:Boolean);
begin
     inherited Create(True);
     API       :=WPAPI;
     Ready     :=CreateEvent(nil,False,False,nil);
     FreeOnTerminate :=True;
     if Ready<>0 then
     begin
          Resume;
          Success:=(WaitForSingleObject(Ready,1000)=WAIT_OBJECT_0);
          CloseHandle(Ready);
     end else Success :=False;
     if not Success then Destroy;
end;

procedure TPluginAPI.DoTerminate;
begin
     API.PluginAPI :=nil;
     inherited DoTerminate;
end;

procedure TPluginAPI.Execute;
var
  Dup:PMsg;
begin
     // Get Thread ID
     FThreadID :=GetCurrentThreadID;
     // Prepare Message queue
     PeekMessage(Msg,0,WM_USER,WM_USER,PM_NOREMOVE);
     SetEvent(Ready);
     // Message handling loop
     repeat
           GetMessage(Msg,0,WP_FIRST,WP_LASTSERVICE);
           case Msg.Message of
           WP_DISCONNECT :PostMessage(Application.MainForm.Handle,WP_DISCONNECT,
                                      Integer(API),0);
           WP_GETINFO    :ReturnInfo(Msg.lParam);
           WP_QUIT       :{We are disconnecting};
           else
             begin
                  // Message should be an command
                  New(Dup);
                  Dup^:=Msg;
                  PostMessage(Application.MainForm.Handle,WP_XCMD,
                              Integer(API),Integer(Dup));
             end;                    
           end;
     until Msg.Message=WP_QUIT;
end;

procedure TPluginAPI.ReturnInfo(Receiver:Integer);
var
  Buffer :TSharedMem;
  Size   :Integer;
  Res    :DWORD;
begin
     Size   :=SizeOf(TAPPInfo);
     Buffer :=TSharedMem.Create('WPPluginInfo'+IntToHex(GetCurrentThreadID,8),
                                Size);
     if Buffer.Created and (Buffer.Buffer<>nil) then
     begin
          Move(API.AppInfo,Buffer.Buffer^,Size);
          SendThreadMessage(Receiver,WP_RESULT,0,WP_SUCCESS,Res)
     end;
     Buffer.Destroy;
end;



{==============================================================================}
{=== TWinPointAPI implementation ==============================================}
{==============================================================================}

constructor TWinPointAPI.Create(AOwner:TComponent);
var
  Success:Boolean;
begin
     inherited Create(AOwner);
     FAutoConn  :=True;
     FConnected :=False;
     PluginAPI  :=TPluginAPI.Create(Self,Success);
     Services   :=TServiceList.Create;
     with APPInfo do
     begin
          Copyright   :='(C) Copyright by the Author';
          Name        :='Custom Plugin';
          Description :='WinPoint Plugin created by a lazy man!';
          Version     :=1;
          Revision    :=0;
          APIReqd     :=CurrentAPIVersion;
          APISupp     :=CurrentAPIVersion;
     end;
end;

destructor TWinPointAPI.Destroy;
begin
     Disconnect;
     // Kill PluginAPI
     PostThreadMessage(PluginAPI.ThreadID,WP_QUIT,0,0);
     while Assigned(PluginAPI) do sleep(50);
     // Remove API from all Services
     Services.Destroy;
     inherited Destroy;
end;

procedure TWinPointAPI.Loaded;
begin
     inherited Loaded;
     if (not(csDesigning in ComponentState)) and AutoConnect then Connect;
end;

procedure TWinPointAPI.Acknowledge;
begin
     if Connected then PostThreadMessage(API,WP_ACK,0,0);
end;

procedure TWinPointAPI.ServerDisconnected;
begin
     if Connected then
     begin
          FConnected:=False;
          if not (csDestroying in ComponentState) then
          if Assigned(OnServerDisconnected) then OnServerDisconnected(Self);
     end;
end;

procedure TWinPointAPI.Disconnecting;
begin
     if not (csDestroying in ComponentState) then
     if Assigned(OnDisconnecting) then OnDisconnecting(Self)
end;

procedure TWinPointAPI.Disconnected;
begin
     if not (csDestroying in ComponentState) then
     if Assigned(OnDisconnected) then OnDisconnected(Self);
end;

procedure TWinPointAPI.Disconnect;
begin
     if Connected then
     begin
          // We disconnect now
          Disconnecting;
          // Notify the Server that we disconnect
          PostThreadMessage(API,WP_DISCONNECT,0,0);
          // Say we are no longer connected
          FConnected:=False;
          // Done
          Disconnected;
     end;
end;

procedure TWinPointAPI.GetInfo(var AppInfo:TAppInfo);
{Exceptions:ENotConnected}
begin
     if Connected
     then GetAppInfo(API,AppInfo)
     else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.HandleCommand(Msg:TMsg);
var
  Handled :Boolean;
  i       :Integer;
begin
     // First try to see if we have a managed Service handling the command
     Handled:=False;
     for i:=0 to Services.Count-1 do
     with Services[i] do
     begin
          if Active and (Command=Msg.Message) then
          begin
               DoCommand(Msg);
               Handled:=True;
               Break;
          end;
     end;
     // If no managed Service handled the Command we pass it to the Event
     if (not Handled) and Assigned(OnCommand) then OnCommand(Msg);
end;

procedure TWinPointAPI.CloseArea;
{Exceptions:
  ENoAreaOpen    if there was no open Area
  EUnknownError  if an undefined Error Occurs
  ENotConnected  if connection to WinPoint is not established }
var
  Result:DWORD;
begin
     if Connected then
     begin
          SendThreadMessage(API,WP_CLOSEAREA,0,0,Result);
          case Result of
          WP_SUCCESS:{everything went fine};
          WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No area open to close');
          else raise EUnknownAPIError.Create('Unknown Error');
          end;
     end else raise ENotConnected.Create('Not connected to WinPoint!');
end;

procedure TWinPointAPI.SetMsgFlags(Index:Integer; NewFlags:Word);
{Exceptions:EBadMsgIndex, EAreaNotOpen, EUnknownAPIError, ENotConnected}
var
  Result:DWORD;
begin
     if Connected then
     begin
          SendThreadMessage(API,WP_SETMSGFLAGS,NewFlags,Index,Result);
          case Result of
          WP_SUCCESS:{everything went fine};
          WPERR_BADMSGINDEX:raise EMsgBadIndex.Create('Messageindex out of Range');
          WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No Area open!');
          else raise EUnknownAPIError.Create('Unknown Error');
          end;
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

function TWinPointAPI.GetMsgFlags(Index:Integer):Word;
{Exceptions:ENotConnected; Ebadmsgindex, EAreaNotOpen, EUnknownAPIError}
var
  Temp:DWORD;
begin
     if Connected then
     begin
          SendThreadMessage(API,WP_GETMSGFLAGS,0,Index,Temp);
          if Temp>$7fffffff then
          begin
               case Temp of
               WPERR_BADMSGINDEX:raise EMsgBadIndex.Create('Messageindex out of Range');
               WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No Area open!');
               else raise EUnknownAPIError.Create('Unknown Error');
               end;
          end else Result:=Temp;
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.Connect;
var
  WPWindow :HWnd;
  i        :Integer;
  Temp     :DWORD;
begin
     if not Connected then
     begin
          if Assigned(OnConnecting) then OnConnecting(Self);
          WPWindow   :=FindWindow('TWPMainWindow',nil);
          if WPWindow<>0 then
          begin
               API:=SendMessage(WPWindow,WP_QUERY_API,
                                PluginAPI.ThreadID,GetCurrentThreadID);
               if API<>0 then
               begin
                    FConnected:=True;
                    if SendThreadMessage(API,WP_GETAPIVERSION,0,0,Temp) then
                    begin
                         FAPIVersion :=Temp;
                         // Verify API Version
                         if FAPIVersion<AppInfo.APIReqd then
                         begin
                              // Disconnect!
                              Disconnect;
                              // Say we have a version mismatch
                              if Assigned(OnWrongVersion)
                              then OnWrongVersion(Self);
                         end else
                         begin
                              // Add managed service
                              for i:=0 to Services.Count-1
                              do RegisterManagedService(Services[i]);
                              // Say were connected
                              if Assigned(OnConnected)
                              then OnConnected(Self);
                         end;
                    end else Disconnect;
               end;
          end;
          if not Connected and Assigned(OnConnectFailed)
          then OnConnectFailed(Self);
     end;
end;

procedure TWinPointAPI.RegisterManagedService(Service:TCustomWPService);
var
  PluginService :TPluginService;
begin
     if Connected and Service.Active then
     begin
          with PluginService do
          begin
               ServiceClass :=Integer(Service.ServiceClass);
               Title        :=Service.Title;
               Description  :=Service.Description;
               Command      :=Service.Command;
               Plugin       :=PluginAPI.ThreadID;
          end;
          Service.Handle:=RegisterService(PluginService);
     end;
end;

procedure TWinPointAPI.DeRegisterManagedService(Service:TCustomWPService);
begin
     if Connected then with Service do if Active then DeRegisterService(Handle);
end;

procedure TWinPointAPI.Manage(Service:TCustomWPService);
begin
     Services.Add(Service);
     RegisterManagedService(Service);
end;

procedure TWinPointAPI.DontManage(Service:TCustomWPService);
begin
     if not (csDestroying in ComponentState) then
     begin
          Services.Remove(Service);
          DeRegisterManagedService(Service);
     end;
end;

procedure TWinPointAPI.SetConnected(NewState:Boolean);
begin
     if FConnected<>NewState then
     begin
          if NewState then Connect else Disconnect;
     end;
end;

function TWinPointAPI.SendStringCommand(Cmd:Cardinal; ID,Data:String;
                                        var Res:DWORD):Boolean;
var
  Buffer :TSharedMem;
  Size   :Integer;
begin
     Data   :=Data+#0;
     Size   :=Length(Data);
     Buffer :=TSharedMem.Create(ID+IntToHex(GetCurrentThreadID,8),Size);
     if Buffer.Created and (Buffer.Buffer<>nil) then
     begin
          Move(Data[1],Buffer.Buffer^,Size);
          Result:=SendThreadMessage(API,CMD,0,Size,Res)
     end else Result:=False;
     Buffer.Destroy;
end;

procedure TWinPointAPI.StoreMsg(NewMail:PNewMail);
// Store new message in that area
{Exceptions: EUnknownAPIError, ECmdFailed, EWrongFormat, ENoAreaOpen,
             ENotConnected}
var
  Buffer :TSharedMem;
  Res    :DWORD;
begin
     if Connected then
     begin
          Buffer:=TSharedMem.Create('WPStoreNew'+IntToHex(GetCurrentThreadID,8),
                                    NewMail^.Size);
          if Buffer.Created and (Buffer.Buffer<>nil) then
          begin
               Move(NewMail^,Buffer.Buffer^,NewMail^.Size);
               SendThreadMessage(API,WP_STOREMSG,0,NewMail^.Size,Res);
               case Res of
               WP_SUCCESS:{fine ;)};
               WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No Area open!');
               WPERR_WRONGFORMAT:raise EWrongFormat.Create('Wrong data format!');
               WPERR_NOPARAM:raise ECmdFailed.Create('Parameter passing failed!');
               else raise EUnknownAPIError.Create('Unexpected API Error!');
               end;
          end else raise ECmdFailed.Create('Parameter passing failed!');
          Buffer.Destroy;
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.CreateArea(AName,AUplink:String; AType:Byte);
// Store new message in that area
{Exceptions: EUnknownAPIError, ECmdFailed, EWrongFormat, EUplinkUnknown,
             ENotConnected}
var
  Buffer  :TSharedMem;
  Res     :DWORD;
begin
     if Connected then
     begin
          Buffer:=TSharedMem.Create('WPCreateArea'+IntToHex(GetCurrentThreadID,8),
                                    SizeOf(TNewArea));
          if Buffer.Created and (Buffer.Buffer<>nil) then
          begin
               // Fill in Data
               with PNewArea(Buffer.Buffer)^ do
               begin
                    Name     :=AName;
                    Uplink   :=AUplink;
                    AreaType :=AType;
               end;
               SendThreadMessage(API,WP_CREATEAREA,0,SizeOf(TNewArea),Res);
               case Res of
               WP_SUCCESS:{fine ;)};
               WPERR_AREAEXISTS:raise EAreaExists.Create('Area already exists!');
               WPERR_WRONGFORMAT:raise EWrongFormat.Create('Wrong data format!');
               WPERR_NOPARAM:raise ECmdFailed.Create('Parameter passing failed!');
               WPERR_UPLINKUNKNOWN:raise EUplinkUnknown.Create('Unknown Uplink!');
               else raise EUnknownAPIError.Create('Unexpected API Error!');
               end;
          end else raise ECmdFailed.Create('Parameter passing failed!');
          Buffer.Destroy;
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.GetUplinks(Uplinks:TNetLinkList);
{exceptions: ENotConnected, ECmdFailed, EUnknownAPIError}
{Note: This function implicitly calls ClearContent() ! }
var
  Buffer :TSharedMem;
  Size   :DWORD;
  Num,i  :Integer;
  Data   :Pointer;
  Neu    :PNetLinkInfo;
begin
     Uplinks.ClearContent;
     if Connected then
     begin
          if SendThreadMessage(API,WP_GETUPLINKS,0,0,Size) then
          begin
               Buffer:=TSharedMem.Create('WPUplinks'+IntToHex(API,8),Size);
               try
                 if Size<=$7fffffff then
                 begin
                      if (not Buffer.Created) and (Buffer.Buffer<>nil) then
                      begin
                           Num  :=Size div SizeOf(TNetLinkInfo);
                           Data :=Buffer.Buffer;
                           for i:=0 to Num-1 do
                           begin
                                New(Neu);
                                Move(Data^,Neu^,SizeOf(TNetLinkInfo));
                                Inc(Integer(Data),SizeOf(TNetLinkInfo));
                                Uplinks.Add(Neu);
                           end;
                      end else raise ECmdFailed.Create('Parameter passing failed!');
                 end else
                 begin
                      case Size of
                      WPERR_NOPARAM:raise ECmdFailed.Create('Parameter passing failed!');
                      else raise EUnknownAPIError.Create('Unexpected API Error!');
                      end;
                 end;
               finally
                 Buffer.Destroy;
                 Acknowledge;
               end;
          end else raise ECmdFailed.Create('Parameter passing failed!');
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.GetMsgList(MsgList:TMessagelist);
{exceptions: ENotConnected, ECmdFailed, EAreaNotOpen, EUnknownAPIError}
{Note: This function implicitly calls ClearContent() ! }
var
  Buffer :TSharedMem;
  Size   :DWORD;
  Num,i  :Integer;
  Data   :Pointer;
  Neu    :PStoredHeader;
begin
     MsgList.ClearContent;
     if Connected then
     begin
          if SendThreadMessage(API,WP_GETMSGLIST,0,0,Size) then
          begin
               Buffer:=TSharedMem.Create('WPMsgList'+IntToHex(API,8),Size);
               try
                 if Size<=$7fffffff then
                 begin
                      if (not Buffer.Created) and (Buffer.Buffer<>nil) then
                      begin
                           Data :=Buffer.Buffer;
                           Num  :=Integer(Data^);
                           Inc(Integer(Data),4);
                           //Get Data Here!
                           for i:=0 to Num-1 do
                           begin
                                New(Neu);
                                Move(Data^,Neu^,SizeOf(TStoredHeader));
                                Inc(Integer(Data),SizeOf(TStoredHeader));
                                MsgList.Add(Neu);
                           end;
                      end else raise ECmdFailed.Create('Parameter passing failed!');
                 end else
                 begin
                      case Size of
                      WPERR_NOPARAM:raise ECmdFailed.Create('Parameter passing failed!');
                      WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No Area open!');
                      else raise EUnknownAPIError.Create('Unexpected API Error!');
                      end;
                 end;
               finally
                 Buffer.Destroy;
                 Acknowledge;
               end;
          end else raise ECmdFailed.Create('Parameter passing failed!');
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

function TWinPointAPI.GetCurrentArea:String;
{exceptions: ENotConnected, ECmdFailed, EAreaNotOpen, EUnknownAPIError}
var
  Buffer :TSharedMem;
  Size   :DWORD;
begin
     Result:='';
     if Connected then
     begin
          if SendThreadMessage(API,WP_GETCURRENTAREA,0,0,Size) then
          begin
               Buffer:=TSharedMem.Create('WPCurrentArea'+IntToHex(API,8),Size);
               try
                 if Size<=$7fffffff then
                 begin
                      if (not Buffer.Created) and (Buffer.Buffer<>nil)
                      then Result :=PChar(Buffer.Buffer)
                      else raise ECmdFailed.Create('Parameter passing failed!');
                 end else
                 begin
                      case Size of
                      WPERR_NOPARAM:ECmdFailed.Create('Parameter passing failed!');
                      WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No Area open!');
                      else raise EUnknownAPIError.Create('Unexpected API Error!');
                      end;
                 end;
               finally
                 Buffer.Destroy;
                 Acknowledge;
               end;
          end else raise ECmdFailed.Create('Parameter passing failed!');
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.SetDialTranslations(Translations:TStrings);
// Store new message in that area
{Exceptions: EUnknownAPIError, ECmdFailed, EWrongFormat,
             ENotConnected}
var
  Buffer :TSharedMem;
  Res    :DWORD;
  i      :Integer;
  Trans  :String;
begin
     if Connected then
     begin
          // Create Data
          Trans:='';
          for i:=0 to Translations.Count-1
          do Trans:=Trans+Translations[i]+#$0d#$0a;
          // Post Data
          Buffer:=TSharedMem.Create('WPNDT'+IntToHex(GetCurrentThreadID,8),
                                    Length(Trans)+1);
          if Buffer.Created and (Buffer.Buffer<>nil) then
          begin
               Move(Trans[1],Buffer.Buffer^,Length(Trans)+1);
               SendThreadMessage(API,WP_SETTRANSLATIONS,0,Length(Trans)+1,Res);
               // Handle Result
               case Res of
               WP_SUCCESS:{fine ;)};
               WPERR_WRONGFORMAT:raise EWrongFormat.Create('Wrong data format!');
               WPERR_NOPARAM:raise ECmdFailed.Create('Parameter passing failed!');
               else raise EUnknownAPIError.Create('Unexpected API Error!');
               end;
          end else raise ECmdFailed.Create('Parameter passing failed!');
          Buffer.Destroy;
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.GetDialTranslations(Translations:TStrings);
{Exceptions: ENotConnected, ECmdFailed, EUnknownError}
var
  Buffer :TSharedMem;
  Size   :DWORD;
  Data   :PChar;
  Txt    :String;
  C      :Char;
begin
     Translations.Clear;
     if Connected then
     begin
          if SendThreadMessage(API,WP_GETTRANSLATIONS,0,0,Size) then
          begin
               if Size>0 then
               begin
                    Buffer:=TSharedMem.Create('WPDT'+IntToHex(API,8),Size);
                    if Size<=$7fffffff then
                    begin
                         if (not Buffer.Created) and (Buffer.Buffer<>nil) then
                         begin
                              Data:=Buffer.Buffer;
                              while (Size>0) and (Data[0]<>#0) do
                              begin
                                   Txt:='';
                                   while (Size>0) and (Data[0]<>#0) do
                                   begin
                                        C:=Data[0];
                                        Inc(Integer(Data));
                                        Dec(Size);
                                        if C=#$0d then
                                        begin
                                             if (Size>1) and (Data[0]=#$0a) then
                                             begin
                                                  Inc(Integer(Data));
                                                  Dec(Size);
                                             end;
                                             Break;
                                        end else Txt:=Txt+C
                                   end;
                                   Translations.Add(Txt);
                              end;
                         end else
                         begin
                              Buffer.Destroy;
                              raise ECmdFailed.Create('Command failed');
                         end;
                         Buffer.Destroy;
                         Acknowledge;
                    end else raise EUnknownAPIError.Create('Unexpected Error!');
               end;
          end else raise ECmdFailed.Create('Command failed');
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

function TWinPointAPI.GetMsg(Index:Integer):PPakdMessage;
{Exceptions: ENotConnected, ECmdFailed, EMsgBadIndex, EUnknownError, EAreaNotOpen}
var
  Buffer :TSharedMem;
  Size   :DWORD;
begin
     Result :=nil;
     if Connected then
     begin
          if SendThreadMessage(API,WP_GETMSG,0,Index,Size) then
          begin
               if Size>0 then
               begin
                    Buffer:=TSharedMem.Create('WPMsg'+IntToHex(API,8),Size);
                    if Size<=$7fffffff then
                    begin
                         if (not Buffer.Created) and (Buffer.Buffer<>nil) then
                         begin
                              GetMem(Result,Size);
                              Move(Buffer.Buffer^,Result^,Size);
                         end else
                         begin
                              Buffer.Destroy;
                              raise ECmdFailed.Create('Command failed');
                         end;
                         Buffer.Destroy;
                         Acknowledge;
                    end else
                    case Size of
                    WPERR_BADMSGINDEX:raise EMsgBadIndex.Create('Message Index out of range!');
                    WPERR_NOAREAOPEN:raise EAreaNotOpen.Create('No Area open!');
                    else raise EUnknownAPIError.Create('Unexpected Error!');
                    end;
               end;
          end else raise ECmdFailed.Create('Command failed');
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.GetAreaList(AreaList:TStrings);
// Assigns list to Arealist
//     Strings contains the AreaName and Objects the Type!
{Exceptions: ENotConnected, ECmdFailed}
var
  Buffer :TSharedMem;
  Size   :DWORD;
  Num,i  :Integer;
  ATyp   :Integer;
  Temp   :String;
  Data   :Pointer;
begin
     AreaList.Clear;
     if Connected then
     begin
          if SendThreadMessage(API,WP_GETAREALIST,0,0,Size) and
             (Size<=$7fffffff) then
          begin
               Buffer:=TSharedMem.Create('WPAreaList'+IntToHex(API,8),Size);
               if (not Buffer.Created) and (Buffer.Buffer<>nil) then
               begin
                    Data :=Buffer.Buffer;
                    Num  :=Integer(Data^);
                    Inc(Integer(Data),4);
                    for i:=0 to Num-1 do
                    begin
                         Temp:=PChar(Data);
                         Inc(Integer(Data),Length(Temp)+1);
                         ATyp:=Byte(Data^);
                         Inc(Integer(Data));
                         AreaList.AddObject(Temp,Pointer(ATyp));
                    end;
               end else
               begin
                    Buffer.Destroy;
                    raise ECmdFailed.Create('Could not get result');
               end;
               Buffer.Destroy;
               Acknowledge;
          end else ECmdFailed.Create('Could not send command!');
     end else ENotConnected.Create('Not connected to WinPoint');
end;

procedure TWinPointAPI.OpenArea(Name:String);
{Exceptions:
 ECmdFailed
 ENotConnected
 EAreaInUse
 EOpenAreaFailed
 EAreaNotFound
 EUnknownError
}
var
  Res:DWORD;
begin
     if Connected then
     begin
          if SendStringCommand(WP_OPENAREA,'WPOpenArea',Name,Res) then
          begin
               case Res of
               WP_SUCCESS:{area open};
               WPERR_NOPARAM:raise ECmdFailed.Create('Could not send parameter!');
               WPERR_AREAINUSE:raise EAreaInUse.Create('Area in use!');
               WPERR_AREAOPEN:raise EAreaOpenFailed.Create('Could not open area!');
               WPERR_AREAUNKNOWN:raise EAreaUnknown.Create('Area not found!');
               else raise EUnknownAPIError.Create('Unknown Error');
               end;
          end else ECmdFailed.Create('Could not send command!');
     end else raise ENotConnected.Create('Not connected to WinPoint!');
end;

function TWinPointAPI.GetAreaType(Name:String):DWORD;
{Exceptions:
 ECmdFailed
 ENotConnected
 EAreaNotFound
 EUnknownError
}
begin
     if Connected then
     begin
          if SendStringCommand(WP_GETAREATYPE,'WPGetAreaType',Name,Result) then
          begin
               if Result>$7fffffff then
               case Result of
               WPERR_NOPARAM:raise ECmdFailed.Create('Could not send parameter!');
               WPERR_AREAUNKNOWN:raise EAreaUnknown.Create('Area not found!');
               else raise EUnknownAPIError.Create('Unknown Error');
               end;
          end else ECmdFailed.Create('Could not send command!');
     end else raise ENotConnected.Create('Not connected to WinPoint!');
end;

procedure TWinPointAPI.GetAreaUplink(Name:String; var Uplink:TNetLinkInfo);
{Exceptions:
 ECmdFailed
 ENotConnected
 EAreaNotFound
 EUnknownError
 ENoUplink
}
var
  Res:DWORD;
  Buffer :TSharedMem;
begin
     if Connected then
     begin
          if SendStringCommand(WP_GETAREAUPLINK,'WPGetAreaUplink',Name,Res) then
          begin
               case Res of
               WP_SUCCESS:
                 begin
                      Buffer:=TSharedMem.Create('WPAreaUplink'+IntToHex(API,8),
                                                SizeOf(Uplink));
                      if (not Buffer.Created) and (Buffer.Buffer<>nil) then
                      begin
                           Move(Buffer.Buffer^,Uplink,SizeOf(Uplink));
                      end else
                      begin
                           Buffer.Destroy;
                           raise ECmdFailed.Create('Command failed');
                      end;
                      Buffer.Destroy;
                      Acknowledge;
                 end;
               WPERR_NOUPLINK:raise ENoUplink.Create('Area '+Name+' has no uplink!');
               WPERR_NOPARAM:raise ECmdFailed.Create('Could not send parameter!');
               WPERR_AREAUNKNOWN:raise EAreaUnknown.Create('Area not found!');
               else raise EUnknownAPIError.Create('Unknown Error');
               end;
          end else raise ECmdFailed.Create('Could not send command!');
     end else raise ENotConnected.Create('Not connected to WinPoint!');
end;

//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================

{==============================================================================}
{=== DeRegisterService(Service) =============================== API Version 1 =}
{==============================================================================}
{ Deregister Plugin Provided Service                                           }
{==============================================================================}
{- In -------------------------------------------------------------------------}
{ lParam as ServiceHandle                                                      }
{   Service to deregister                                                      }
{- Out ------------------------------------------------------------------------}
{ APIResult as LongBool                                                        }
{   Returns true if the service was registered                                 }
{- Description ----------------------------------------------------------------}
{ This function has to be called when the Service described by "Servicehandle" }
{ is no longer available                                                       }
{- Notes ----------------------------------------------------------------------}
{ Each  call  to   this   function   MUST   have   a   coresponding   call  to }
{ DeRegisterService(Service) where "Service" is the result of this function    }
{==============================================================================}
procedure TWinPointAPI.DeRegisterService(Service:Integer);
{Exceptions; ENotConnected, EUnknownError, EServiceUnknown}
var
  Res :DWORD;
begin
     if Connected then
     begin
          if SendThreadMessage(API,WP_DEREGISTERSERVICE,0,Service,Res) then
          case Res of
          WP_SUCCESS:{well done!};
          WPERR_SERVICEUNKNOWN:raise EServiceUnknown.Create('Unknown Service');
          else raise EUnknownAPIError.Create('Unknown Error');
          end;
     end else raise ENotConnected.Create('Not connected to WinPoint');
end;

{==============================================================================}
{=== RegisterService(Service) ================================= API Version 1 =}
{==============================================================================}
{ Register Plugin Provided Service                                             }
{==============================================================================}
{- In -------------------------------------------------------------------------}
{ Memory Mapped File 'WPRegisterServiceXXXXXXXX' as TPluginService             }
{   Service to register                                                        }
{- Out ------------------------------------------------------------------------}
{ APIResult as ServiceHandle                                                   }
{   Handle for registered Service or 0 if failed                               }
{- Description ----------------------------------------------------------------}
{ Registering a Service means  making a plugin  function available to the user }
{ in the "Plugins" menus WinPoint has in most of its windows. The value of the }
{ ServiceClass entry of the "Service" parameter  determines in which menus the }
{ Service will be available.  If the ServiceClass  is SC_General,  the Service }
{ will be available in all "Plugins" menus.                                    }
{- Notes ----------------------------------------------------------------------}
{ Each  call  to   this   function   MUST   have   a   coresponding   call  to }
{ DeRegisterService(Service) where "Service" is the result of this function    }
{==============================================================================}
function TWinPointAPI.RegisterService(Service:TPluginService):HService;
{Exceptions: ENotConnected; ECmdFailed}
var
  Temp :String;
  Res  :DWORD;
begin
     if Connected then
     begin
          Service.Plugin :=PluginAPI.ThreadID;
          SetLength(Temp,sizeOf(TPluginService));
          Move(Service,Temp[1],sizeOf(TPluginService));
          if SendStringCommand(WP_REGISTERSERVICE,
                                    'WPRegisterService',Temp,Res)
          then Result:=Res
          else raise ECmdFailed.Create('Could not transmit command');
     end else raise ENotConnected.Create('Not connected to WinPoint!');
end;

class procedure TWinPointAPI.HandleMsgs(var Msg:TMsg; var Handled:Boolean);
begin
     case Msg.Message of
       WP_DISCONNECT:
         begin
              with (TObject(Msg.wParam) as TWinPointAPI)
              do ServerDisconnected;
              Handled:=True;
         end;
       WP_XCMD:
         begin
              with (TObject(Msg.wParam) as TWinPointAPI) do
              HandleCommand(PMsg(Msg.lParam)^);
              Dispose(PMsg(Msg.lParam));
              Handled:=True;
         end;
       else if Assigned(OldOnMessage) then OldOnMessage(Msg,Handled);
     end;
end;

//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================

procedure Register;
begin
     RegisterComponents('WinPoint', [TWinPointAPI]);
     RegisterComponents('WinPoint', [TWinPointService]);
     RegisterComponents('WinPoint', [TAreaService]);
     RegisterComponents('WinPoint', [TMsgService]);
end;

initialization
OldOnMessage          :=Application.OnMessage;
Application.OnMessage :=TWinpointAPI.HandleMsgs;

finalization
Application.OnMessage :=OldOnMessage;

end.
